@@ -1,8 +1,10 @@
 """Cloudnet product quality checks."""
 import dataclasses
 import datetime
+import json
 import logging
 import os
+import re
 from dataclasses import dataclass
 from enum import Enum
 from pathlib import Path
@@ -13,7 +15,7 @@ import scipy.stats
 from numpy import ma
 
 from . import utils
-from .variables import VARIABLES, Product
+from .variables import LEVELS, VARIABLES, Product
 from .version import __version__
 
 DATA_PATH = os.path.join(os.path.dirname(__file__), "data")
@@ -94,18 +96,14 @@ def run_tests(
 def test(
     name: str,
     description: str,
-    error_level: ErrorLevel | None = None,
     products: list[Product] | None = None,
     ignore_products: list[Product] | None = None,
 ):
     """Decorator for the tests."""
 
     def fun(cls):
-
         setattr(cls, "name", name)
         setattr(cls, "description", description)
-        if error_level is not None:
-            setattr(cls, "severity", error_level)
         if products is not None:
             setattr(cls, "products", [member.value for member in products])
         if ignore_products is not None:
@@ -121,7 +119,6 @@ class Test:
 
     name: str
     description: str
-    severity = ErrorLevel.WARNING
     products: list[str] = Product.all()
 
     def __init__(self, nc: netCDF4.Dataset, filename: Path, cloudnet_file_type: str):
@@ -136,14 +133,23 @@ class Test:
     def run(self):
         raise NotImplementedError
 
-    def _add_message(self, message: str | list):
+    def _add_message(self, message: str | list, severity: ErrorLevel):
         self.report.exceptions.append(
             {
                 "message": utils.format_msg(message),
-                "result": self.severity,
+                "result": severity,
             }
         )
 
+    def _add_info(self, message: str | list):
+        self._add_message(message, ErrorLevel.INFO)
+
+    def _add_warning(self, message: str | list):
+        self._add_message(message, ErrorLevel.WARNING)
+
+    def _add_error(self, message: str | list):
+        self._add_message(message, ErrorLevel.ERROR)
+
     def _read_config_keys(self, config_section: str) -> np.ndarray:
         field = "all" if "attr" in config_section else self.cloudnet_file_type
         keys = METADATA_CONFIG[config_section][field].split(",")
@@ -165,11 +171,15 @@ class Test:
             if key not in VARIABLES:
                 continue
             expected = getattr(VARIABLES[key], attribute)
+            if callable(expected):
+                expected = expected(self.nc)
             if expected is not None:
                 value = getattr(self.nc.variables[key], attribute, "")
                 if value != expected:
-                    msg = utils.create_expected_received_msg(key, expected, value)
-                    self._add_message(msg)
+                    msg = utils.create_expected_received_msg(
+                        expected, value, variable=key
+                    )
+                    self._add_warning(msg)
 
     def _get_date(self):
         date_in_file = [int(getattr(self.nc, x)) for x in ("year", "month", "day")]
@@ -181,31 +191,31 @@ class Test:
 # --------------------#
 
 
-@test("Variable outliers", "Find suspicious data values.", error_level=ErrorLevel.INFO)
+@test("Variable outliers", "Find suspicious data values.")
 class FindVariableOutliers(Test):
     def run(self):
         for key, limits_str in DATA_CONFIG.items("limits"):
+            if key == "zenith_angle" and self.cloudnet_file_type.startswith("mwr-"):
+                continue
             limits = [float(x) for x in limits_str.split(",")]
             if key in self.nc.variables:
                 data = self.nc.variables[key][:]
                 if data.ndim > 0 and len(data) == 0:
-                    self.severity = ErrorLevel.ERROR
                     break
                 max_value = np.max(data)
                 min_value = np.min(data)
                 if min_value < limits[0]:
                     msg = utils.create_out_of_bounds_msg(key, *limits, min_value)
-                    self._add_message(msg)
+                    self._add_info(msg)
                 if max_value > limits[1]:
                     msg = utils.create_out_of_bounds_msg(key, *limits, max_value)
-                    self._add_message(msg)
+                    self._add_info(msg)
 
 
 @test(
     "Radar folding",
     "Test for radar folding.",
     products=[Product.RADAR, Product.CATEGORIZE],
-    error_level=ErrorLevel.INFO,
 )
 class FindFolding(Test):
     def run(self):
@@ -214,53 +224,77 @@ class FindFolding(Test):
         try:
             data = self.nc[key][:]
         except IndexError:
-            self.severity = ErrorLevel.ERROR
-            self._add_message(f"Doppler velocity, '{key}', is missing.")
+            self._add_error(f"Doppler velocity, '{key}', is missing.")
             return
         difference = np.abs(np.diff(data, axis=1))
         n_suspicious = ma.sum(difference > v_threshold)
         if n_suspicious > 20:
-            self._add_message(f"{n_suspicious} suspicious pixels. Folding might be present.")
+            self._add_info(
+                f"{n_suspicious} suspicious pixels. Folding might be present."
+            )
 
 
 @test(
     "Data coverage",
     "Test that file contains enough data.",
-    ignore_products=[Product.MODEL],
-    error_level=ErrorLevel.INFO,
 )
 class TestDataCoverage(Test):
+    RESOLUTIONS = {
+        Product.MODEL: datetime.timedelta(hours=1),
+        Product.MWR_MULTI: datetime.timedelta(minutes=20),
+        Product.WEATHER_STATION: datetime.timedelta(minutes=1),
+        Product.DISDROMETER: datetime.timedelta(minutes=1),
+    }
+    DEFAULT_RESOLUTION = datetime.timedelta(seconds=30)
+
     def run(self):
-        date_in_file = self._get_date()
-        now = datetime.datetime.now(tz=datetime.timezone.utc)
-        current_fraction_hour = (
-            now.hour + int(now.minute) / 60 if now.date() == date_in_file else 24
-        )
-        grid = ma.array(np.linspace(0, current_fraction_hour, int(24 * (60 / 5)) + 1))
         time = self.nc["time"][:]
-        bins_with_no_data = 0
-        for ind, t in enumerate(grid[:-1]):
-            ind2 = np.where((time > t) & (time <= grid[ind + 1]))[0]
-            if len(ind2) == 0:
-                bins_with_no_data += 1
-        missing = bins_with_no_data / len(grid) * 100
+        time_unit = datetime.timedelta(hours=1)
+        try:
+            n_time = len(time)
+        except (TypeError, ValueError):
+            return
+        if n_time == 0:
+            return
+        expected_res = self.RESOLUTIONS.get(
+            self.cloudnet_file_type, self.DEFAULT_RESOLUTION
+        )
+        now = datetime.datetime.now(tz=datetime.timezone.utc)
+        if now.date() == self._get_date():
+            midnight = now.replace(hour=0, minute=0, second=0, microsecond=0)
+            duration = now - midnight
+        else:
+            duration = datetime.timedelta(days=1)
+        bins = max(1, duration // expected_res)
+        hist, _bin_edges = np.histogram(
+            time, bins=bins, range=(0, duration / time_unit)
+        )
+        missing = np.count_nonzero(hist == 0) / len(hist) * 100
         if missing > 20:
+            message = f"{round(missing)}% of day's data is missing."
             if missing > 60:
-                self.severity = ErrorLevel.WARNING
-            self._add_message(f"{round(missing)}% of day's data is missing.")
+                self._add_warning(message)
+            else:
+                self._add_info(message)
+
+        actual_res = np.median(np.diff(time)) * time_unit
+        if actual_res > expected_res * 1.05:
+            self._add_warning(
+                f"Expected a measurement with interval at least {expected_res},"
+                f" got {actual_res} instead"
+            )
 
 
 @test(
     "Variable names",
     "Check that variables have expected names.",
     ignore_products=[Product.MODEL],
-    error_level=ErrorLevel.INFO,
 )
 class TestVariableNamesDefined(Test):
     def run(self):
         for key in self.nc.variables.keys():
             if key not in VARIABLES:
-                self._add_message(f"'{key}' is not defined in cloudnetpy-qc.")
+                self._add_info(f"'{key}' is not defined in cloudnetpy-qc.")
 
 
 # ---------------------- #
@@ -305,64 +339,136 @@ class TestDataTypes(Test):
             if received != expected:
                 if key == "time" and received in ("float32", "float64"):
                     continue
-                msg = utils.create_expected_received_msg(key, expected, received)
-                self._add_message(msg)
+                msg = utils.create_expected_received_msg(
+                    expected, received, variable=key
+                )
+                self._add_warning(msg)
+
+
+# @test(
+#     "Time data type",
+#     "Check that time vector is in double precision.",
+#     products=[
+#         Product.RADAR,
+#         Product.LIDAR,
+#         Product.MWR,
+#         Product.DISDROMETER,
+#         Product.WEATHER_STATION,
+#     ],
+# )
+# class TestTimeVectorDataType(Test):
+#     def run(self):
+#         key = "time"
+#         received = self.nc.variables[key].dtype.name
+#         expected = "float64"
+#         if received != expected:
+#             msg = utils.create_expected_received_msg(key, expected, received)
+#             self._add_warning(msg)
 
 
 @test("Global attributes", "Check that file contains required global attributes.")
 class TestGlobalAttributes(Test):
+    REQUIRED_ATTRS = {
+        "year",
+        "month",
+        "day",
+        "file_uuid",
+        "Conventions",
+        "location",
+        "history",
+        "title",
+        "cloudnet_file_type",
+        "source",
+    }
+
+    def _instrument_product(self, product: Product):
+        return (
+            LEVELS[product] == "1b" and product != Product.MODEL
+        ) or product == Product.MWR_L1C
+
+    def _required_attrs(self, product: Product):
+        attrs = set(self.REQUIRED_ATTRS)
+        if product == Product.MWR_L1C:
+            attrs.add("mwrpy_coefficients")
+        if product != Product.MODEL:
+            attrs.add(
+                "instrument_pid"
+                if self._instrument_product(product)
+                else "source_file_uuids"
+            )
+        return attrs
+
+    def _optional_attr(self, name: str, product: Product) -> bool:
+        return (
+            name == "references"
+            or name.endswith("_version")
+            or (
+                product == Product.MODEL
+                and name in ("initialization_time", "institution")
+            )
+            or (self._instrument_product(product) and name == "serial_number")
+        )
+
     def run(self):
-        nc_keys = self.nc.ncattrs()
-        config_keys = self._read_config_keys("required_global_attributes")
-        missing_keys = list(set(config_keys) - set(nc_keys))
+        nc_keys = set(self.nc.ncattrs())
+        required_attrs = self._required_attrs(self.cloudnet_file_type)
+        missing_keys = required_attrs - nc_keys
         for key in missing_keys:
-            self._add_message(f"'{key}' is missing.")
+            self._add_warning(f"Attribute '{key}' is missing.")
+        extra_keys = nc_keys - required_attrs
+        for key in extra_keys:
+            if not self._optional_attr(key, self.cloudnet_file_type):
+                self._add_warning(f"Unknown attribute '{key}' found.")
 
 
 @test(
     "Median LWP",
     "Test that median of LWP values is within a reasonable range.",
-    ErrorLevel.WARNING,
     [Product.MWR, Product.CATEGORIZE],
 )
 class TestMedianLwp(Test):
     def run(self):
         key = "lwp"
         if key not in self.nc.variables:
-            self.severity = ErrorLevel.ERROR
-            self._add_message(f"'{key}' is missing.")
+            self._add_error(f"'{key}' is missing.")
             return
         limits = [-0.5, 10]
         median_lwp = ma.median(self.nc.variables[key][:]) / 1000  # g -> kg
         if median_lwp < limits[0] or median_lwp > limits[1]:
             msg = utils.create_out_of_bounds_msg(key, *limits, median_lwp)
-            self._add_message(msg)
+            self._add_warning(msg)
 
 
 @test("Attribute outliers", "Find suspicious values in global attributes.")
 class FindAttributeOutliers(Test):
     def run(self):
-        for key, limits_str in METADATA_CONFIG.items("attribute_limits"):
-            limits = [float(x) for x in limits_str.split(",")]
-            if hasattr(self.nc, key):
-                value = float(self.nc.getncattr(key))
-                if value < limits[0] or value > limits[1]:
-                    msg = utils.create_out_of_bounds_msg(key, *limits, value)
-                    self._add_message(msg)
+        try:
+            year = int(self.nc.year)
+            month = int(self.nc.month)
+            day = int(self.nc.day)
+            datetime.date(year, month, day)
+        except AttributeError:
+            self._add_warning("Missing some date attributes.")
+        except ValueError:
+            self._add_warning("Invalid date attributes.")
 
 
 @test(
-    "LDR values", "Test that LDR values are proper.", products=[Product.RADAR, Product.CATEGORIZE]
+    "LDR values",
+    "Test that LDR values are proper.",
+    products=[Product.RADAR, Product.CATEGORIZE],
 )
 class TestLDR(Test):
     def run(self):
         if "ldr" in self.nc.variables:
             ldr = self.nc["ldr"][:]
             if ldr.mask.all():
-                self._add_message("LDR exists but all the values are invalid.")
+                self._add_warning("LDR exists but all the values are invalid.")
 
 
-@test("Range correction", "Test that beta is range corrected.", products=[Product.LIDAR])
+@test(
+    "Range correction", "Test that beta is range corrected.", products=[Product.LIDAR]
+)
 class TestIfRangeCorrected(Test):
     def run(self):
         try:
@@ -374,25 +480,23 @@ class TestIfRangeCorrected(Test):
         x = range_var[-n_top_ranges:] ** 2
         y = np.std(beta_raw[:, -n_top_ranges:], axis=0)
         res = scipy.stats.pearsonr(x, y)
-        if res.statistic < 0.5:
-            self._add_message("Data might not be range corrected.")
+        if res.statistic < 0.75:
+            self._add_warning("Data might not be range corrected.")
 
 
 @test(
-    "Instrument PID",
-    "Test that valid instrument PID exists.",
-    ErrorLevel.WARNING,
-    [Product.MWR, Product.LIDAR, Product.RADAR, Product.DISDROMETER],
+    "Floating-point values",
+    "Test for special floating-point values which may indicate problems with the processing.",
 )
-class TestInstrumentPid(Test):
+class TestFloatingPointValues(Test):
     def run(self):
-        key = "instrument_pid"
-        try:
-            pid = getattr(self.nc, key)
-            if pid == "":
-                self._add_message("Instrument PID is empty.")
-        except AttributeError:
-            self._add_message("Instrument PID is missing.")
+        for name, variable in self.nc.variables.items():
+            if variable.dtype.kind != "f":
+                continue
+            if np.isnan(variable[:]).any():
+                self._add_warning(f"Variable '{name}' contains NaN value(s).")
+            if np.isinf(variable[:]).any():
+                self._add_warning(f"Variable '{name}' contains infinite value(s).")
 
 
 # ---------------------#
@@ -403,7 +507,6 @@ class TestInstrumentPid(Test):
 @test(
     "Beta presence",
     "Test that one beta variable exists.",
-    error_level=ErrorLevel.ERROR,
     products=[Product.LIDAR],
 )
 class TestLidarBeta(Test):
@@ -412,43 +515,43 @@ class TestLidarBeta(Test):
         for key in valid_keys:
             if key in self.nc.variables:
                 return
-        self._add_message("No valid beta variable found.")
+        self._add_error("No valid beta variable found.")
 
 
-@test("Time vector", "Test that time vector is continuous.", ErrorLevel.ERROR)
+@test("Time vector", "Test that time vector is continuous.")
 class TestTimeVector(Test):
     def run(self):
         time = self.nc["time"][:]
         try:
             n_time = len(time)
         except (TypeError, ValueError):
-            self._add_message("Time vector is empty.")
+            self._add_error("Time vector is empty.")
             return
         if n_time == 0:
-            self._add_message("Time vector is empty.")
+            self._add_error("Time vector is empty.")
             return
         if n_time == 1:
-            self._add_message("One time step only.")
+            self._add_error("One time step only.")
             return
         differences = np.diff(time)
         min_difference = np.min(differences)
         max_difference = np.max(differences)
         if min_difference <= 0:
             msg = utils.create_out_of_bounds_msg("time", 0, 24, min_difference)
-            self._add_message(msg)
+            self._add_error(msg)
         if max_difference >= 24:
             msg = utils.create_out_of_bounds_msg("time", 0, 24, max_difference)
-            self._add_message(msg)
+            self._add_error(msg)
 
 
-@test("Variables", "Check that file contains required variables.", ErrorLevel.ERROR)
+@test("Variables", "Check that file contains required variables.")
 class TestVariableNames(Test):
     def run(self):
         keys_in_file = set(self.nc.variables.keys())
         required_keys = self._get_required_variable_names()
         missing_keys = list(required_keys - keys_in_file)
         for key in missing_keys:
-            self._add_message(f"'{key}' is missing.")
+            self._add_error(f"'{key}' is missing.")
 
 
 # ------------------------------#
@@ -475,11 +578,158 @@ class TestCFConvention(Test):
                 if not error_msg:
                     continue
                 if level in ("FATAL", "ERROR"):
-                    self.severity = ErrorLevel.ERROR
+                    severity = ErrorLevel.ERROR
                 elif level == "WARN":
-                    self.severity = ErrorLevel.WARNING
+                    severity = ErrorLevel.WARNING
                 else:
                     continue
                 msg = utils.format_msg(error_msg)
                 msg = f"Variable '{key}': {msg}"
-                self._add_message(msg)
+                self._add_message(msg, severity)
+
+
+@test(
+    "Instrument PID",
+    "Test that valid instrument PID exists.",
+    [
+        Product.MWR,
+        Product.LIDAR,
+        Product.RADAR,
+        Product.DISDROMETER,
+        Product.DOPPLER_LIDAR,
+        Product.WEATHER_STATION,
+    ],
+)
+class TestInstrumentPid(Test):
+    data: dict = {}
+
+    def run(self):
+        if self._check_exists():
+            self.data = utils.fetch_pid(self.nc.instrument_pid)
+            self._check_serial()
+            self._check_model_name()
+            self._check_model_identifier()
+
+    def _check_exists(self) -> bool:
+        key = "instrument_pid"
+        try:
+            pid = getattr(self.nc, key)
+            if pid == "":
+                self._add_error("Instrument PID is empty.")
+                return False
+            if re.fullmatch(utils.PID_FORMAT, pid) is None:
+                self._add_error("Instrument PID has invalid format.")
+                return False
+        except AttributeError:
+            self._add_warning("Instrument PID is missing.")
+            return False
+        return True
+
+    def _get_value(self, kind: str) -> dict | list | None:
+        try:
+            item = next(item for item in self.data["values"] if item["type"] == kind)
+            return json.loads(item["data"]["value"])
+        except StopIteration:
+            return None
+
+    def _check_serial(self):
+        key = "serial_number"
+        try:
+            received = str(getattr(self.nc, key))
+        except AttributeError:
+            return
+        items = self._get_value("21.T11148/eb3c713572f681e6c4c3")
+        if not isinstance(items, list):
+            return
+        model_name = self._get_value("21.T11148/c1a0ec5ad347427f25d6")["modelName"]
+        for item in items:
+            if item["alternateIdentifier"]["alternateIdentifierType"] == "SerialNumber":
+                expected = item["alternateIdentifier"]["alternateIdentifierValue"]
+                if "StreamLine" in model_name:
+                    expected = expected.split("-")[-1]
+                if received != expected:
+                    msg = utils.create_expected_received_msg(expected, received)
+                    self._add_error(msg)
+                return
+        self._add_warning(
+            f"No serial number was defined in instrument PID but found '{received}' in the file."
+        )
+
+    def _check_model_name(self):
+        key = "source"
+        try:
+            source = getattr(self.nc, key)
+            allowed_values = self.SOURCE_TO_NAME[source]
+        except (AttributeError, KeyError):
+            return
+        model = self._get_value("21.T11148/c1a0ec5ad347427f25d6")
+        if model is None:
+            return
+        received = model["modelName"]
+        if received not in allowed_values:
+            msg = utils.create_expected_received_msg(allowed_values, received)
+            self._add_error(msg)
+
+    def _check_model_identifier(self):
+        key = "source"
+        try:
+            source = getattr(self.nc, key)
+            allowed_values = self.SOURCE_TO_IDENTIFIER[source]
+        except (AttributeError, KeyError):
+            return
+        model = self._get_value("21.T11148/c1a0ec5ad347427f25d6")
+        if model is None:
+            return
+        if "modelIdentifier" not in model:
+            return
+        received = model["modelIdentifier"]["modelIdentifierValue"]
+        if received not in allowed_values:
+            msg = utils.create_expected_received_msg(allowed_values, received)
+            self._add_error(msg)
+
+    SOURCE_TO_NAME = {
+        "Lufft CHM15k": ["Lufft CHM 15k"],
+        "Lufft CHM15kx": ["Lufft CHM 15k-x"],
+        "TROPOS PollyXT": ["PollyXT"],
+        "Vaisala CL31": ["Vaisala CL31"],
+        "Vaisala CL51": ["Vaisala CL51"],
+        "Vaisala CL61d": ["Vaisala CL61"],
+        "Vaisala CT25k": ["Vaisala CT25K"],
+        "HALO Photonics StreamLine": [
+            "StreamLine",
+            "StreamLine Pro",
+            "StreamLine XR",
+            "StreamLine XR+",
+        ],
+    }
+
+    SOURCE_TO_IDENTIFIER = {
+        "BASTA": ["https://vocabulary.actris.nilu.no/actris_vocab/BASTA"],
+        "METEK MIRA-35": [
+            "https://vocabulary.actris.nilu.no/actris_vocab/METEKMIRA35",
+            "https://vocabulary.actris.nilu.no/actris_vocab/METEKMIRA35S",
+        ],
+        "OTT HydroMet Parsivel2": [
+            "https://vocabulary.actris.nilu.no/actris_vocab/OTTParsivel2"
+        ],
+        "RAL Space Copernicus": [
+            "https://vocabulary.actris.nilu.no/actris_vocab/UFAMCopernicus"
+        ],
+        "RAL Space Galileo": [
+            "https://vocabulary.actris.nilu.no/actris_vocab/UFAMGalileo"
+        ],
+        "RPG-Radiometer Physics HATPRO": [
+            "https://vocabulary.actris.nilu.no/actris_vocab/RPGHATPRO"
+        ],
+        "RPG-Radiometer Physics RPG-FMCW-35": [
+            "https://vocabulary.actris.nilu.no/skosmos/actris_vocab/en/page/RPG-FMCW-35-DP"
+            "https://vocabulary.actris.nilu.no/skosmos/actris_vocab/en/page/RPG-FMCW-35-SP"
+            "https://vocabulary.actris.nilu.no/skosmos/actris_vocab/en/page/RPG-FMCW-35S"
+        ],
+        "RPG-Radiometer Physics RPG-FMCW-94": [
+            "https://vocabulary.actris.nilu.no/actris_vocab/RPG-FMCW-94-DP",
+            "https://vocabulary.actris.nilu.no/actris_vocab/RPG-FMCW-94-SP",
+            "https://vocabulary.actris.nilu.no/actris_vocab/RPG-FMCW-94S",
+        ],
+        "Thies Clima LNM": ["https://vocabulary.actris.nilu.no/actris_vocab/ThiesLNM"],
+    }
